import es, popuplib, cmdlib, cfglib
import random

info = es.AddonInfo()
info['name']        = "CSSDM Style Guns Menu"
info['version']     = "1.1.1"
info['author']      = "DanielB" 
info['url']         = "http://forums.eventscripts.com/viewtopic.php?t=34141" 
info['description'] = "CSSDM Guns menu for DanielBs Advanced Deathmatch" 

dm = es.import_addon('dm')

if not dm.minver('2.1.5'): # This script uses dm.fire_event
    raise NotImplementedError, 'Please update your dm version'
           
d_players = {} # {userid: GunSelection_instance}
weapons = {'primary'   : ['m4a1', 'ak47', 'awp', 'scout', 'sg552', 'galil', 'famas',
                          'sg550', 'aug', 'm249', 'g3sg1', 'ump45', 'mp5navy', 'm3',
                           'xm1014', 'tmp', 'mac10', 'p90'],
           'secondary' : ['usp', 'deagle', 'glock', 'p228', 'elite', 'fiveseven'] }
           
           
popupmap = {#'name':     ('popup_name', line_no),
            'm4a1':      ('dm_guns_primary1', 2),
            'ak47':      ('dm_guns_primary1', 3),
            'awp':       ('dm_guns_primary1', 4),
            'scout':     ('dm_guns_primary1', 5),
            'sg552':     ('dm_guns_primary1', 6),
            'galil':     ('dm_guns_primary1', 7),
            'famas':     ('dm_guns_primary1', 8),
            'sg550':     ('dm_guns_primary2', 2),
            'aug':       ('dm_guns_primary2', 3),
            'm249':      ('dm_guns_primary2', 4),
            'g3sg1':     ('dm_guns_primary2', 5),
            'ump45':     ('dm_guns_primary2', 6),
            'mp5navy':   ('dm_guns_primary2', 7),
            'm3':        ('dm_guns_primary2', 8),
            'xm1014':    ('dm_guns_primary3', 2),
            'tmp':       ('dm_guns_primary3', 3),
            'mac10':     ('dm_guns_primary3', 4),
            'p90':       ('dm_guns_primary3', 5),
            'usp':       ('dm_guns_secondary', 2),
            'deagle':    ('dm_guns_secondary', 4),
            'glock':     ('dm_guns_secondary', 3),
            'p228':      ('dm_guns_secondary', 5),
            'elite':     ('dm_guns_secondary', 6),
            'fiveseven': ('dm_guns_secondary', 7)
            }
            
class Settings(object):
    config = cfglib.AddonCFG(dm.addondir + '/dm/addons/cssdmguns/settings.cfg')
    
    for weapon_name in popupmap:
        cvar_name = 'dm_cssdmguns_allow_%s'%weapon_name
        config.cvar(cvar_name, '1', 'Allow this weapon? (1=allow,0=disallow)')
        es.ServerVar(cvar_name).addFlag('notify')
    
    config.execute()
    
    def __init__(self):
        self.config.write()
        
    def __getattr__(self, name):
        if hasattr(Settings, name):
            return super(Settings, self).__getattribute__(name)
           
        elif hasattr(Settings, '_'+name):
            val, func = super(Settings, self).__getattribute__('_'+name)
            return func(val)
           
        raise AttributeError, 'No such attribute "%s"'%name
           
           
class RandomList:
    def __init__(self, listitems):
        self.items = listitems
    def __str__(self):
        unrestricted = filter(lambda x: int(es.ServerVar('dm_cssdmguns_allow_%s'%x)), self.items)
        if unrestricted:
            return random.choice(unrestricted)
        return ''
        
randomPrimary   = RandomList(weapons['primary'])
randomSecondary = RandomList(weapons['secondary'])
    

class GunSelection:
    def __init__(self, userid):
        self.userid = int(userid)
        
        self.primary      = False
        self.secondary    = False
        self.keepsame     = False
        self.accessedmenu = False
        
    def has_made_selections(self):
        return bool(self.primary and self.secondary)
        
    def give_weapons(self):
        self.fire('player_weaponstrip kill')
        self.give('player_weaponstrip')
        self.fire('player_weaponstrip strip')
        
        self.give('weapon_%s'%self.primary, 0.1)
        self.give('weapon_%s'%self.secondary,  0.1)
        self.give('weapon_knife', 0.1)
        
        evs = {'userid': ['int', self.userid], 'prestripped': ['int', 1] }
        dm.fire_event('dm_player_loadout', evs)
        
    def fire(self, strcmd, delay=-1):
        if delay > 0:
            es.delayed(delay, 'es_xfire %s %s'%(self.userid, strcmd))
        else:
            es.server.queuecmd('es_xfire %s %s'%(self.userid, strcmd))
        
    def give(self, itemname, delay=-1):
        if delay > 0:
            es.delayed(delay, 'es_xgive %s %s'%(self.userid, itemname))
        else:
            es.server.queuecmd('es_xgive %s %s'%(self.userid, itemname))
        

def load():
    cmdlib.registerSayCommand('guns', cmd_guns, 'Gun selection menu')
    for weapon in popupmap:
        server_cvar( {'cvarname': 'dm_cssdmguns_allow_' + weapon} )
    
    dm.msg('Loaded Addon - CSS:DM Style Guns Menu')
    
def unload():
    for p in [p_main, p_secondary, p_primary1, p_primary2, p_primary3]:
        p.delete()
    cmdlib.unregisterSayCommand('guns')
    
    dm.msg('Unloaded Addon - CSS:DM Style Guns Menu')
    
def server_cvar(event_var):
    cvar_name = event_var['cvarname']
    
    if cvar_name.startswith('dm_cssdmguns_allow_'):
        weapon_name = cvar_name.replace('dm_cssdmguns_allow_', '')
        cvar = es.ServerVar(cvar_name)
        
        popupname, lineno = popupmap[weapon_name]
        
        popup = popuplib.find(popupname)
        
        #es.dbgmsg(0, 'editing %s'%weapon_name)
        #es.dbgmsg(0, popup.lines[popup.editlang])
        
        linetext = popup.lines[popup.editlang][lineno]
        
        #es.dbgmsg(0, linetext)
        
        if int(cvar):
            if not linetext.startswith('->'):
                popup.modline(lineno, '->' + linetext)
                #es.dbgmsg(0, 'newline : ->' + linetext)
        else:
            if linetext.startswith('->'):
                popup.modline(lineno, linetext[2:])
                #es.dbgmsg(0, 'newline : ' + linetext[2:])
                
def has_popup_open(userid):
    popups = ['dm_guns_main', 'dm_guns_secondary', 'dm_guns_primary1',
    'dm_guns_primary2', 'dm_guns_primary3']
    
    for p in popups:
        if popuplib.isqueued(p, userid):
            return True
    
    return False
    
def is_allowed(weapon_name):
    if not isinstance(weapon_name, RandomList):
        cvar_name = 'dm_cssdmguns_allow_' + weapon_name
        cvar = es.ServerVar(cvar_name)
    
        return int(cvar)
    return True
    
    
def cmd_guns(userid, args):
    global d_players
    userid = int(userid)
    
    if not userid in d_players:
        d_players[userid] = GunSelection(userid)
    
    
    if not d_players[userid].accessedmenu:
        d_players[userid].accessedmenu = True
        sendmain(userid)
    else:
        dm.tell(userid, 'Your guns menu is already enabled')
        
        
        
def sendmain(userid):
    global d_players
    
    if es.isbot(userid):
        botsetup = GunSelection(userid)
        botsetup.primary   = randomPrimary 
        botsetup.secondary = randomSecondary
        botsetup.keepsame  = True
        d_players[userid] = botsetup
        return
    
    
    if not userid in d_players:
        d_players[userid] = GunSelection(userid)
    
    if d_players[userid].has_made_selections():
        p_main.modline(3, '->2. Same weapons')
        p_main.modline(4, '->3. Same weapons every time')
    else:
        p_main.modline(3, '2. Same weapons')
        p_main.modline(4, '3. Same weapons every time')
        
    p_main.send(userid)
    
def p_main_select(userid, choice, popupname):
    userid = int(userid)
    
    if not userid in d_players:
        d_players[userid] = GunSelection(userid)
    
    if choice == 1:
        p_secondary.send(userid)
        d_players[userid].keepsame = False
        
    elif choice == 2:
        selection = d_players[userid]
        if userid in d_players and d_players[userid].has_made_selections():
            d_players[userid].give_weapons()
            d_players[userid].keepsame = False
        else:
            sendmain(userid)
        
    elif choice == 3:
        selection = d_players[userid]
        if userid in d_players and d_players[userid].has_made_selections():
            d_players[userid].keepsame = True
            d_players[userid].give_weapons()
        else:
            sendmain(userid)
        
    elif choice == 4:
        d_players[userid].primary   = randomPrimary
        d_players[userid].secondary = randomSecondary
        d_players[userid].keepsame  = False
        d_players[userid].give_weapons()
        
    elif choice == 5:
        d_players[userid].primary   = randomPrimary
        d_players[userid].secondary = randomSecondary
        d_players[userid].keepsame  = True
        d_players[userid].give_weapons()
        
    else: # invalid
    
        sendmain(userid)
        
def p_secondary_select(userid, choice, popupname):
    userid = int(userid)
    
    if 0 < choice < 8: # If valid choice
    
        secondary = [None, 'usp', 'glock', 'deagle', 'p228', 'elite', 'fiveseven', randomSecondary]
        
        selection = secondary[choice]
        if is_allowed(selection):
            d_players[userid].secondary = selection
            p_primary1.send(userid)
        else:
            dm.tell(userid, 'That weapon is restricted')
            p_secondary.send(userid)
        
    elif choice == 10:
        d_players[userid].accessedmenu = False
        
    else: # Invalid!
        p_secondary.send(userid)
        
def p_primaryx_select(userid, choice, popupname):
    userid = int(userid)
    
    if 0 < choice < 8:
        menu_setup = {
        'dm_guns_primary1': [None, 'm4a1', 'ak47', 'awp', 'scout', 'sg552', 'galil', 'famas'],
        'dm_guns_primary2': [None, 'sg550', 'aug', 'm249', 'g3sg1', 'ump45', 'mp5navy', 'm3'],
        'dm_guns_primary3': [None, 'xm1014', 'tmp', 'mac10', 'p90', randomPrimary, None, None]
        }           
            
        selection = menu_setup[popupname][choice]
        if selection:
            if is_allowed(selection):
                d_players[userid].primary = selection
                d_players[userid].give_weapons()
            else:
                dm.tell(userid, 'That weapon is restricted')
                popuplib.find(popupname).send(userid)
        else:
            popuplib.find(popupname).send(userid)
       
    
    elif choice == 8: # previous
        {'dm_guns_primary1': p_primary1, 'dm_guns_primary2': p_primary1,
        'dm_guns_primary3': p_primary2}[popupname].send(userid)
        
    elif choice == 9: # next
        {'dm_guns_primary1': p_primary2, 'dm_guns_primary2': p_primary3,
        'dm_guns_primary3': p_primary3}[popupname].send(userid)
        
    elif choice == 10:
        d_players[userid].accessedmenu = False
        
    else: # Invalid!
        popuplib.find(popupname).send(userid)
        
        
def player_spawn(ev):
    userid = int(ev['userid'])
    
    if es.getplayerteam(userid) in (2,3):
        if userid in d_players:
            selection = d_players[userid]
            d_players[userid].accessedmenu = False
            if selection.has_made_selections() and selection.keepsame:
                selection.give_weapons()
            else:
                sendmain(userid)
            
        else:
             sendmain(userid)
         
def player_disconnect(ev):
    global d_players
    userid = int(ev['userid'])
    
    if userid in d_players:
        del d_players[userid]
        
def es_map_start(ev):
    d_players.clear()
    
p_main = popuplib.create('dm_guns_main')
p_main.addline('Weapon Options:')
p_main.addline('->1. New weapons')
p_main.addline('->2. Same weapons')
p_main.addline('->3. Same weapons every time')
p_main.addline('->4. Random weapons')
p_main.addline('->5. Random weapons every time')
p_main.menuselect = p_main_select

p_secondary = popuplib.create('dm_guns_secondary')
p_secondary.addline('Seconadary weapon:')
p_secondary.addline('->1. USP')
p_secondary.addline('->2. Glock')
p_secondary.addline('->3. Deagle')
p_secondary.addline('->4. P228')
p_secondary.addline('->5. Elite')
p_secondary.addline('->6. Five Seven')
p_secondary.addline('->7. Random')
p_secondary.addline(' ')
p_secondary.addline('0. Exit')
p_secondary.menuselect = p_secondary_select

p_primary1 = popuplib.create('dm_guns_primary1')
p_primary1.addline('Primary Weapon:')
p_primary1.addline('->1. M4A1')
p_primary1.addline('->2. AK47')
p_primary1.addline('->3. AWP')
p_primary1.addline('->4. Scout')
p_primary1.addline('->5. SG552')
p_primary1.addline('->6. Galil')
p_primary1.addline('->7. Famas')
p_primary1.addline(' ')
p_primary1.addline(' ')
p_primary1.addline('->9. Next')
p_primary1.addline('0. Exit')
p_primary1.menuselect = p_primaryx_select

p_primary2 = popuplib.create('dm_guns_primary2')
p_primary2.addline('Primary Weapon:')
p_primary2.addline('->1. SG550')
p_primary2.addline('->2. Steyr AUG')
p_primary2.addline('->3. M249')
p_primary2.addline('->4. G3SG1')
p_primary2.addline('->5. UMP 45')
p_primary2.addline('->6. MP5 Navy')
p_primary2.addline('->7. M3')
p_primary2.addline(' ')
p_primary2.addline('->8. Previous')
p_primary2.addline('->9. Next')
p_primary2.addline('0. Exit')
p_primary2.menuselect = p_primaryx_select

p_primary3 = popuplib.create('dm_guns_primary3')
p_primary3.addline('Primary Weapon:')
p_primary3.addline('->1. xm1014')
p_primary3.addline('->2. TMP')
p_primary3.addline('->3. Mac 10')
p_primary3.addline('->4. P90')
p_primary3.addline('->5. Random')
p_primary3.addline(' ')
p_primary3.addline(' ')
p_primary3.addline(' ')
p_primary3.addline('->8. Previous')
p_primary3.addline(' ')
p_primary3.addline('0. Exit')
p_primary3.menuselect = p_primaryx_select


settings = Settings() # Needs to be at the bottom so the popups exist