import es, cfglib, playerlib
import time

info = es.AddonInfo()
info['name']        = "EndGame"
info['version']     = "1.0.0"
info['author']      = "DanielB" 
info['url']         = "http://forums.eventscripts.com/viewtopic.php?t=34141" 
info['description'] = "Ends the round when certain conditions are met"

dm = es.import_addon('dm')

class Settings(object):
    config = cfglib.AddonCFG(dm.addondir + '/dm/addons/endgame/settings.cfg')
    
    _limitby  = config.cvar('dm_limitby', 'both', 'End round by what condition? (frags,time,both)'), str
    
    config.text('The following variables should be set in your default config (such as autoexec.exe or server.cfg)')
    
    config.text('mp_fraglimit 150 // Maximum number of kills (when using frags or both)')
    config.text('mp_timelimit 60  // Time until map change in minutes (when using time or both)')
    
    config.execute()
    
    def __init__(self):
        self.config.write()
        
    def __getattr__(self, name):
        if hasattr(Settings, name):
            return super(Settings, self).__getattribute__(name)
           
        elif hasattr(Settings, '_'+name):
            val, func = super(Settings, self).__getattribute__('_'+name)
            return func(val)
           
        raise AttributeError, 'No such attribute "%s"'%name
        
settings = Settings()

last_map_start = time.time()

mp_fraglimit = es.ServerVar('mp_fraglimit')
mp_timelimit = es.ServerVar('mp_timelimit')

def load():
    dm.msg('Loaded Addon - End Game')
    
    if settings.limitby in ('time', 'both'):
        if mp_timelimit == 0:
            mp_timelimit.set(60) # apply a default value
    if settings.limitby in ('frags', 'both'):
        if mp_fraglimit == 0:
            mp_fraglimit.set(100) # apply a default value
    
def unload():
    dm.msg('Unloaded Addon - End Game')
    
def getRemainString():
    timeleft = float(mp_timelimit) * 60 - (time.time() - last_map_start)
    if timeleft <= 0:
        return 'with the next death'
    else:
        minutes, seconds = divmod(timeleft, 60)
        if minutes:
            return 'in %i minutes, %i seconds'%(minutes, seconds)
        else:
            return 'in %i seconds'%seconds
    
def es_map_start(event_var):
    global last_map_start
    last_map_start = time.time()
    
def player_say(event_var):
    if event_var['text'] == 'timeleft':
        method = settings.limitby
        
        if method == 'time':
            es.msg('Round will end ' + getRemainString())
        if method == 'frags':
            es.msg('Round will end when somebody reaches %i kills!'%mp_fraglimit)
        elif method == 'both':
            es.msg('round will end ' + getRemainString() + \
              ' or when somebody reaches %i kills!'%mp_fraglimit) 
        
def player_death(event_var):
    method = settings.limitby
    userid = event_var['userid']
    
    if method in ('time', 'both'):
        if time.time() - last_map_start >= float(mp_timelimit) * 60:
            es.centertell('Game Over!')
            es.server.queuecmd("es_xgive %s game_end"%userid)
            es.server.queuecmd("es_xfire %s game_end endgame"%userid)
        
    if method in ('frags', 'both'):
        player = playerlib.getPlayer(userid)
        if player.kills >= int(mp_fraglimit):
            es.centertell('%s wins!'%(player.name))
            es.server.queuecmd("es_xgive %s game_end"%userid)
            es.server.queuecmd("es_xfire %s game_end endgame"%userid)
    



    
